@@ -32,6 +32,9 @@
 
 #define B32(x)	__be32_to_cpu((x))
 
+/* maximum allowed HP boot loader (IPL) size */
+#define MAXBLSIZE (256 * 1024)
+
 static int Install = 0;
 int verbose = 0;
 
@@ -94,7 +97,7 @@ check_bootloader(int media, int line)
 	error(6, "ipl_addr");
 
     if (B32(f.ipl_size) <= 0 ||
-	B32(f.ipl_size) >= 256 * 1024 ||
+	B32(f.ipl_size) >= MAXBLSIZE ||
 	(B32(f.ipl_size) % 2048) != 0)
 	error(6, "ipl_size");
 
@@ -148,16 +151,13 @@ check_bootloader(int media, int line)
 	B32(f.kern32_offset), B32(f.kern32_sz), B32(f.kern32_native_sz),
 	B32(f.kern64_offset), B32(f.kern64_sz), B32(f.kern64_native_sz),
 	B32(f.rd_offset), B32(f.rd_sz),
-	f.cmdline);
+	f.cmdline[0] ? f.cmdline : f.cmdline_old);
 #endif
 
     return 1;
 }
 
 
-/* maximum allowed HP boot loader (IPL) size */
-#define MAXBLSIZE (256 * 1024)
-
 unsigned
 write_bootloader(int media, int bootloader,
 	unsigned where,
@@ -307,7 +307,7 @@ do_sequential(int media, int kernel32, int kernel64,
     where += write(media, &f, FW_BLOCKSIZE - (where % FW_BLOCKSIZE));
 
     if (commandline != 0)
-	strcpy(f.cmdline, commandline);
+	strncpy(f.cmdline, commandline, sizeof(f.cmdline)-1);
 
     STRUCTWRITE(media, f, 0);
 
@@ -427,7 +427,7 @@ do_cdrom(int media, int kernel32, int kernel64,
     }
 
     if (commandline != 0)
-	strcpy(f.cmdline, commandline);
+	strncpy(f.cmdline, commandline, sizeof(f.cmdline)-1);
 
     STRUCTWRITE(media, f, 0);
 
@@ -523,7 +523,7 @@ do_formatted(int init, int media, const char *medianame, int partition,
 	f.flags |= PFLAG_EXT2;
 
 	if(commandline)
-	    strncpy(f.cmdline, commandline, sizeof(f.cmdline));
+	    strncpy(f.cmdline, commandline, sizeof(f.cmdline)-1);
 
 	write_bootloader(media, bootloaderfd, holestart,
 			 holestart + EXT2_HOLE*EXT2_BLOCKSIZE, &f);
@@ -544,7 +544,7 @@ do_formatted(int init, int media, const char *medianame, int partition,
 	}
 
 	if(commandline)
-	    strncpy(f.cmdline, commandline, sizeof(f.cmdline));
+	    strncpy(f.cmdline, commandline, sizeof(f.cmdline)-1);
 
 	write_bootloader(media, bootloaderfd, f.ipl_addr,
 			 f.ipl_addr + EXT2_HOLE*EXT2_BLOCKSIZE, &f);
@@ -621,7 +621,7 @@ do_randomaccess(int init, int media, int kernel32, int kernel64,
 	    where = write_ramdisk(media, ramdisk, where, end, &f);
 
 	if (commandline != 0)
-	    strcpy(f.cmdline, commandline);
+	    strncpy(f.cmdline, commandline, sizeof(f.cmdline)-1);
 
 	STRUCTWRITE(media, f, 0);
     }
@@ -661,7 +661,7 @@ do_randomaccess(int init, int media, int kernel32, int kernel64,
 	}
 
 	if (commandline != 0)
-	    strcpy(f.cmdline, commandline);
+	    strncpy(f.cmdline, commandline, sizeof(f.cmdline)-1);
 
 	STRUCTWRITE(media, f, 0);
 #endif
@@ -714,7 +714,7 @@ main(int argc, char *argv[])
     char *medianame = NULL;
     char *bootloaderfile = "/usr/share/palo/iplboot";
     int init = 0;
-    extern char *bld_info;
+    extern const char bld_info[];
     const char gargs[] = "f:C:s:b:k:c:r:I:e:U:v?";
     char *config_file = "/etc/palo.conf";
     char *newargv[MAXARGS];
@@ -852,8 +852,8 @@ main(int argc, char *argv[])
 	    break;
 	case 'c':
 	    commandline = optarg;
-	    if (strlen(commandline) > 127)
-		error(3,127,strlen(commandline));
+	    if (strlen(commandline) >= CMDLINELEN)
+		error(3,CMDLINELEN-1,strlen(commandline));
 	    break;
 	case 'e':
 	    if(strcmp(optarg, "2") == 0)
